/* eslint-disable handle-callback-err  */
import React, { Fragment } from 'react'
import { get } from 'lodash'
import styled from 'styled-components'
import { reduxForm } from 'redux-form'
import { th } from '@pubsweet/ui-toolkit'
import { required as requiredValidator } from 'xpub-validators'
import { compose, withStateHandlers, withProps } from 'recompose'
import { H3, Spinner, ValidatedField, TextField, Menu } from '@pubsweet/ui'
import { withCountries } from 'pubsweet-component-faraday-ui'

import {
  Row,
  Item,
  Text,
  Label,
  ActionLink,
  IconButton,
  ShadowedBox,
  withFetching,
  RowOverrideAlert,
} from './'

const Profile = ({
  title,
  toggleEdit,
  user: { affiliation, firstName, lastName, title: userTitle, country = '' },
  countryLabel,
  ...rest
}) => (
  <ShadowedBox position="relative" {...rest}>
    <IconButton
      icon="edit-2"
      iconSize={2}
      onClick={toggleEdit}
      right={8}
      top={12}
    />

    <Fragment>
      <Row alignItems="center">
        <Item>
          <H3>Account Details</H3>
        </Item>
        <Item>
          <ActionLink internal to="/profile/change-password">
            Change password
          </ActionLink>
        </Item>
      </Row>

      <Row alignItems="baseline" mt={2}>
        <Item vertical>
          <Label>First Name</Label>
          <Text secondary>{firstName}</Text>
        </Item>
        <Item vertical>
          <Label>Last Name</Label>
          <Text secondary>{lastName}</Text>
        </Item>
      </Row>

      <Row alignItems="baseline" mt={2}>
        <Item vertical>
          <Label>Title</Label>
          <Text secondary>
            {get(title.find(t => t.value === userTitle), 'label', '')}
          </Text>
        </Item>
        <Item vertical>
          <Label>Country</Label>
          <Text secondary>{countryLabel(country)}</Text>
        </Item>
      </Row>

      <Row alignItems="baseline" mt={2}>
        <Item vertical>
          <Label>Affiliation</Label>
          <Text secondary>{affiliation}</Text>
        </Item>
      </Row>
    </Fragment>
  </ShadowedBox>
)

const EditModeIcons = ({ toggleEdit, onSaveChanges }) => (
  <Fragment>
    <IconButton
      icon="x-circle"
      iconSize={2}
      onClick={toggleEdit}
      right={36}
      top={12}
    />
    <IconButton
      icon="check-circle"
      iconSize={2}
      onClick={onSaveChanges}
      right={8}
      top={12}
    />
  </Fragment>
)

const EditUserProfile = compose(
  withFetching,
  withProps(
    ({
      user: { affiliation, firstName, lastName, title: userTitle, country },
    }) => ({
      initialValues: {
        country,
        lastName,
        firstName,
        affiliation,
        title: userTitle,
      },
    }),
  ),
  reduxForm({
    form: 'profile',
    onSubmit: (values, dispatch, { onSave, ...props }) => {
      typeof onSave === 'function' && onSave(values, props)
    },
  }),
)(
  ({
    toggleEdit,
    title,
    handleSubmit,
    isFetching,
    countries,
    fetchingError,
    ...rest
  }) => (
    <ShadowedBox position="relative" {...rest}>
      {isFetching ? (
        <StyledSpinner>
          <Spinner />
        </StyledSpinner>
      ) : (
        <EditModeIcons onSaveChanges={handleSubmit} toggleEdit={toggleEdit} />
      )}
      <Fragment>
        <Row>
          <Item>
            <H3>Edit Account Details</H3>
          </Item>
        </Row>

        <Row alignItems="baseline" mt={2}>
          <Item mr={1} vertical>
            <Label required>First Name</Label>
            <ValidatedField
              component={TextField}
              name="firstName"
              validate={[requiredValidator]}
            />
          </Item>
          <Item ml={1} vertical>
            <Label required>Last Name</Label>
            <ValidatedField
              component={TextField}
              name="lastName"
              validate={[requiredValidator]}
            />
          </Item>
        </Row>

        <RowOverrideAlert alignItems="baseline" mt={2}>
          <Item mr={1} vertical>
            <Label required>Title</Label>
            <ValidatedField
              component={input => <Menu {...input} options={title} />}
              name="title"
              validate={[requiredValidator]}
            />
          </Item>
          <Item ml={1} vertical>
            <Label required>Country</Label>
            <ValidatedField
              component={input => <Menu {...input} options={countries} />}
              name="country"
              validate={[requiredValidator]}
            />
          </Item>
        </RowOverrideAlert>

        <Row alignItems="baseline" mt={2}>
          <Item vertical>
            <Label required>Affiliation</Label>
            <ValidatedField
              component={TextField}
              name="affiliation"
              validate={[requiredValidator]}
            />
          </Item>
        </Row>

        {fetchingError && (
          <Row alignItems="center" mt={2}>
            <Text error>{fetchingError}</Text>
          </Row>
        )}
      </Fragment>
    </ShadowedBox>
  ),
)

const UserProfile = ({
  user,
  onSave,
  editMode,
  toggleEdit,
  journal: { title = [] },
  countries,
  countryLabel,
  ...rest
}) =>
  !editMode ? (
    <Profile
      countryLabel={countryLabel}
      title={title}
      toggleEdit={toggleEdit}
      user={user}
      {...rest}
    />
  ) : (
    <EditUserProfile
      countries={countries}
      onSave={onSave}
      title={title}
      toggleEdit={toggleEdit}
      user={user}
      {...rest}
    />
  )

export default compose(
  withCountries,
  withStateHandlers(
    {
      editMode: false,
    },
    {
      toggleEdit: ({ editMode }) => () => ({
        editMode: !editMode,
      }),
    },
  ),
)(UserProfile)

// #region styles
const StyledSpinner = styled.div`
  position: absolute;
  top: ${th('gridUnit')};
  right: ${th('gridUnit')};
`
// #endregion
