const config = require('config')
const { get } = require('lodash')
const Email = require('@pubsweet/component-email-templating')

const {
  User,
  services,
  Fragment,
} = require('pubsweet-component-helper-service')

const resetPath = config.get('invite-reset-password.url')
const { name: journalName, staffEmail } = config.get('journal')
const unsubscribeSlug = config.get('unsubscribe.url')

const { getEmailCopy } = require('./emailCopy')

module.exports = {
  async sendNotifications({
    baseUrl,
    fragment,
    UserModel,
    collection,
    isNewVersion = false,
    previousVersion,
    isTechnicalCheck,
    isMajorRecommendation,
  }) {
    const fragmentHelper = new Fragment({ fragment })
    const handlingEditor = get(collection, 'handlingEditor')
    const parsedFragment = await fragmentHelper.getFragmentData({
      handlingEditor,
    })
    const {
      submittingAuthor,
      activeAuthors: authors,
    } = await fragmentHelper.getAuthorData({
      UserModel,
    })

    const { customId } = collection

    const email = new Email({
      type: 'user',
      fromEmail: `${journalName} <${staffEmail}>`,
      content: {
        signatureName: '',
        signatureJournal: journalName,
        ctaLink: services.createUrl(
          baseUrl,
          `/projects/${collection.id}/versions/${fragment.id}/details`,
        ),
        ctaText: 'MANUSCRIPT DETAILS',
      },
    })

    const userHelper = new User({ UserModel })
    const eicName = await userHelper.getEiCName()

    if (isNewVersion) {
      const heUser = await UserModel.find(handlingEditor.id)
      sendHandlingEditorEmail({
        email,
        baseUrl,
        customId,
        title: parsedFragment.title,
        handlingEditor: heUser,
      })
    }

    if (isMajorRecommendation) {
      sendReviewersEmail({
        email,
        baseUrl,
        customId,
        UserModel,
        previousVersion,
        title: parsedFragment.title,
        heName: collection.handlingEditor.name,
      })
    }

    if (isTechnicalCheck) {
      sendEQSEmail({
        email,
        eicName,
        baseUrl,
        collection,
      })
      sendAuthorsEmail({
        email,
        baseUrl,
        UserModel,
        fragmentId: fragment.id,
        fragmentAuthors: authors,
        collectionId: collection.id,
        title: parsedFragment.title,
        submittingAuthorName: `${submittingAuthor.firstName} ${
          submittingAuthor.lastName
        }`,
      })
    }
  },
}

const sendHandlingEditorEmail = ({
  email,
  title,
  baseUrl,
  customId,
  handlingEditor,
}) => {
  const emailType = 'he-new-version-submitted'

  email.toUser = {
    email: handlingEditor.email,
  }
  email.content.unsubscribeLink = services.createUrl(baseUrl, unsubscribeSlug, {
    id: handlingEditor.id,
    token: handlingEditor.accessTokens.unsubscribe,
  })
  email.content.subject = `${customId}: Revision submitted`

  const { html, text } = email.getNotificationBody({
    emailBodyProps: getEmailCopy({
      emailType,
      titleText: `the manuscript titled "${title}"`,
    }),
  })
  email.sendEmail({ html, text })
}

const sendReviewersEmail = async ({
  email,
  title,
  heName,
  baseUrl,
  customId,
  UserModel,
  previousVersion,
}) => {
  email.content.subject = `${customId}: A manuscript you reviewed has been revised`
  email.content.signatureName = heName
  email.fromEmail = `${heName} <${staffEmail}>`
  const emailType = 'submitted-reviewers-after-revision'

  const fragmentHelper = new Fragment({ fragment: previousVersion })
  const reviewers = await fragmentHelper.getReviewers({
    UserModel,
    type: 'submitted',
  })

  reviewers.forEach(reviewer => {
    email.toUser = {
      email: reviewer.email,
      name: `${reviewer.lastName}`,
    }

    email.content.unsubscribeLink = services.createUrl(
      baseUrl,
      unsubscribeSlug,
      {
        id: reviewer.id,
        token: reviewer.accessTokens.unsubscribe,
      },
    )

    const { html, text } = email.getNotificationBody({
      emailBodyProps: getEmailCopy({
        emailType,
        titleText: `the manuscript titled "${title}"`,
        expectedDate: services.getExpectedDate({ daysExpected: 14 }),
      }),
    })

    email.sendEmail({ html, text })
  })
}

const sendEQSEmail = ({ email, eicName, baseUrl, collection }) => {
  const emailType = 'eqs-manuscript-submitted'

  email.toUser = {
    email: staffEmail,
    name: 'Editorial Assistant',
  }

  email.content.unsubscribeLink = baseUrl
  email.content.signatureName = eicName
  email.content.subject = `Manuscript Submitted`
  email.content.ctaLink = services.createUrl(
    baseUrl,
    config.get('eqs-decision.url'),
    {
      collectionId: collection.id,
      customId: collection.customId,
      token: collection.technicalChecks.token,
    },
  )
  email.content.ctaText = 'MAKE DECISION'

  const { html, text } = email.getNotificationBody({
    emailBodyProps: getEmailCopy({
      emailType,
      customId: collection.customId,
    }),
  })
  email.sendEmail({ html, text })
}

const sendAuthorsEmail = async ({
  email,
  title,
  baseUrl,
  UserModel,
  fragmentId,
  collectionId,
  fragmentAuthors,
  submittingAuthorName,
}) => {
  email.content.subject = `Manuscript submitted to ${journalName}`
  email.content.signatureName = ''

  const userEmailData = await Promise.all(
    fragmentAuthors.map(async author => {
      const user = await UserModel.find(author.id)
      return {
        ...author,
        isConfirmed: user.isConfirmed,
        unsubscribeToken: user.accessTokens.unsubscribe,
        passwordResetToken: user.accessTokens.passwordReset,
        ...getEmailCopy({
          emailType: author.isSubmitting
            ? 'submitting-author-manuscript-submitted'
            : 'coauthors-manuscript-submitted',
          titleText: author.isSubmitting
            ? `the manuscript titled "${title}"`
            : `The manuscript titled "${title}" has been submitted to ${journalName} by ${submittingAuthorName}`,
        }),
      }
    }),
  )

  userEmailData.forEach(author => {
    email.toUser = {
      email: author.email,
      name: `${author.lastName}`,
    }
    email.content.unsubscribeLink = services.createUrl(
      baseUrl,
      unsubscribeSlug,
      {
        id: author.id,
        token: author.unsubscribeToken,
      },
    )

    if (author.isSubmitting) {
      email.content.ctaLink = services.createUrl(
        baseUrl,
        `/projects/${collectionId}/versions/${fragmentId}/details`,
      )
      email.content.ctaText = 'MANUSCRIPT DETAILS'
    } else if (author.isConfirmed) {
      email.content.ctaLink = services.createUrl(baseUrl, '')
      email.content.ctaText = 'LOG IN'
    } else {
      email.content.ctaLink = services.createUrl(baseUrl, resetPath, {
        email: author.email,
        token: author.passwordResetToken,
        firstName: author.firstName,
        lastName: author.lastName,
        affiliation: author.affiliation,
        title: author.title,
        country: author.country,
      })
      email.content.ctaText = 'CONFIRM ACCOUNT'
    }

    const { html, text } = email.getNotificationBody({
      emailBodyProps: {
        paragraph: author.paragraph,
        hasLink: author.hasLink,
        hasIntro: author.hasIntro,
        hasSignature: author.hasSignature,
      },
    })
    email.sendEmail({ html, text })
  })
}
